/*
   DVR, a Digital video Recorder for Linux.
   
   Copyright (C) 2001-2007 Pierre Hebert <pierrox@pierrox.net>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this program; if not, write to the
   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.

 */

#include <limits.h>
#include <gst/gst.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qspinbox.h>
#include <qlineedit.h>
#include <qwidgetstack.h>
#include <qradiobutton.h>
#include <qcombobox.h>
#include <qvalidator.h>

#include "property_editor.h"
class QListBoxEnum : public QListBoxText {
  public:
    gint value;
    QListBoxEnum(QListBox *listbox, const QString &text, gint _value) : QListBoxText(listbox, text), value(_value) {}
};

PropertyEditor::PropertyEditor(QWidget *parent, GstElement *_element, const char *_property_name) : 
  UIPropertyEditor(parent, "property_editor", true),
  element(_element),
  property_name(_property_name)
{
  GParamSpec *param;
  QString hint, value;
  GValue prop_value={ 0, 0 };

  param=g_object_class_find_property(G_OBJECT_GET_CLASS(element), property_name);
  g_value_init(&prop_value, param->value_type);
  g_object_get_property(G_OBJECT(element), param->name, &prop_value);

  qlb_property_name->setText(g_param_spec_get_blurb(param));
  
  switch(G_VALUE_TYPE(&prop_value)) {
    case G_TYPE_STRING :
      GParamSpecString *p=G_PARAM_SPEC_STRING(param);
      const char *s=g_value_get_string(&prop_value);
      if(s==NULL) s="";
      qle_string_value->setText(s);
      hint.sprintf("Default value is \"<b>%s</b>\"", p->default_value?p->default_value:"<i>empty</i>");
      qlb_string_hint->setText(hint);
      ws_value->raiseWidget(0);
      break;
      
    case G_TYPE_BOOLEAN : {
      GParamSpecBoolean *p=G_PARAM_SPEC_BOOLEAN(param);
      if(g_value_get_boolean(&prop_value)) {
        qrb_bool_yes->setChecked(true);
      } else {
        qrb_bool_no->setChecked(true);
      }
      hint.sprintf("Default is <b>%s</b>", p->default_value?"yes":"no");
      qlb_bool_hint->setText(hint);
      ws_value->raiseWidget(1);
      break;
    }
      
    case G_TYPE_ULONG : {
      GParamSpecULong *p=G_PARAM_SPEC_ULONG(param);
      value.sprintf("%lu", g_value_get_ulong(&prop_value));
      hint.sprintf("Value from %lu to %lu (default is <b>%lu</b>)", p->minimum, p->maximum, p->default_value);
      setNumberValue(value, hint, NULL /* TODO new QULongValidator(this)*/);
      break;  
    }
    
    case G_TYPE_LONG : {
      GParamSpecLong *p=G_PARAM_SPEC_LONG(param);
      value.sprintf("%ld", g_value_get_long(&prop_value));
      hint.sprintf("Value from %ld to %ld (default is <b>%ld</b>)", p->minimum, p->maximum, p->default_value);
      setNumberValue(value, hint, NULL /* TODO new QLongValidator(this)*/);
      break;  
    }
    
    case G_TYPE_UINT : {
      GParamSpecUInt *p=G_PARAM_SPEC_UINT(param);
      value.sprintf("%u", g_value_get_uint(&prop_value));
      hint.sprintf("Value from %u to %u (default is <b>%u</b>)", p->minimum, p->maximum, p->default_value);
      setNumberValue(value, hint, NULL /* TODO new QUIntValidator(this)*/);
      break;
    }

    case G_TYPE_INT : {
      GParamSpecInt *p=G_PARAM_SPEC_INT(param);
      value.sprintf("%d", g_value_get_int(&prop_value));
      hint.sprintf("Value from %d to %d (default is <b>%d</b>)", p->minimum, p->maximum, p->default_value);
      setNumberValue(value, hint, new QIntValidator(this));
      break;
    }

    case G_TYPE_UINT64 : {
      GParamSpecUInt64 *p=G_PARAM_SPEC_UINT64(param);
      value.sprintf("%"G_GUINT64_FORMAT, g_value_get_uint64(&prop_value));
      hint.sprintf("Value from %"G_GUINT64_FORMAT" to %"G_GUINT64_FORMAT" (default is <b>%"G_GUINT64_FORMAT"</b>)", p->minimum, p->maximum, p->default_value);
      setNumberValue(value, hint, NULL /* TODO new QUInt64Validator(this)*/);
      break;
    }

    case G_TYPE_INT64 : {
      GParamSpecInt64 *p=G_PARAM_SPEC_INT64(param);
      value.sprintf("%"G_GINT64_FORMAT, g_value_get_int64(&prop_value));
      hint.sprintf("Value from %"G_GINT64_FORMAT" to %"G_GINT64_FORMAT" (default is <b>%"G_GINT64_FORMAT"</b>)", p->minimum, p->maximum, p->default_value);
      setNumberValue(value, hint, NULL /* TODO new QInt64Validator(this)*/);
      break;
    }

    case G_TYPE_FLOAT : {
      GParamSpecFloat *p=G_PARAM_SPEC_FLOAT(param);
      value.sprintf("%.7g", g_value_get_float(&prop_value));
      hint.sprintf("Value from %.7g to %.7g (default is <b>%.7g</b>)", p->minimum, p->maximum, p->default_value);
      setNumberValue(value, hint, new QDoubleValidator(this));
      break;
    }

    case G_TYPE_DOUBLE : {
      GParamSpecDouble *p=G_PARAM_SPEC_DOUBLE(param);
      value.sprintf("%.7g", g_value_get_double(&prop_value));
      hint.sprintf("Value from %.7g to %.7g (default is <b>%.7g</b>)", p->minimum, p->maximum, p->default_value);
      setNumberValue(value, hint, new QDoubleValidator(this));
      break;
    }
      
    default:
      if(G_IS_PARAM_SPEC_ENUM(param)) {
        guint j;
        gint value;
        GEnumValue *values;
        GParamSpecEnum *p=G_PARAM_SPEC_ENUM(param);
        
        j=0;
        value=g_value_get_enum (&prop_value);
        values=G_ENUM_CLASS(g_type_class_ref(param->value_type))->values;
        while (values[j].value_name) {
          new QListBoxEnum(qcmb_enum_value->listBox(), values[j].value_name, values[j].value);
          if(values[j].value==value) {
            qcmb_enum_value->setCurrentItem(j);
          }
          if(values[j].value==p->default_value) {
            hint.sprintf("Default value is <b>%s</b>", values[j].value_name);
          }
          j++;
        }
        qlb_enum_hint->setText(hint);
        ws_value->raiseWidget(3);
      } else if(G_IS_PARAM_SPEC_FLAGS(param)) {
        ws_value->raiseWidget(5);
      } else {
        ws_value->raiseWidget(5);
      }
      break;
  }
}

PropertyEditor::~PropertyEditor() {
}

void PropertyEditor::accept() {
  GParamSpec *param;
  param=g_object_class_find_property(G_OBJECT_GET_CLASS(element), property_name);
  switch(G_PARAM_SPEC_VALUE_TYPE(param)) {
    case G_TYPE_STRING :
      g_object_set(G_OBJECT(element), property_name, qle_string_value->text().latin1(), NULL);
      break;
      
    case G_TYPE_BOOLEAN :
      g_object_set(G_OBJECT(element), property_name, qrb_bool_yes->isChecked()?TRUE:FALSE, NULL);
      break;

    case G_TYPE_ULONG : {
      unsigned long v;
      if(sscanf(qlb_number_value->text().latin1(), "%lu", &v)==1) {
        g_object_set(G_OBJECT(element), property_name, v, NULL);
      }
      break;
    }

    case G_TYPE_LONG : {
      long v;
      if(sscanf(qlb_number_value->text().latin1(), "%ld", &v)==1) {
        g_object_set(G_OBJECT(element), property_name, v, NULL);
      }
      break;
    }

    case G_TYPE_UINT : {
      unsigned int v;
      if(sscanf(qlb_number_value->text().latin1(), "%u", &v)==1) {
        g_object_set(G_OBJECT(element), property_name, v, NULL);
      }
      break;
    }

    case G_TYPE_INT : {
      int v;
      if(sscanf(qlb_number_value->text().latin1(), "%d", &v)==1) {
        g_object_set(G_OBJECT(element), property_name, v, NULL);
      }
      break;
    }

    case G_TYPE_UINT64 : {
      unsigned long long v;
      if(sscanf(qlb_number_value->text().latin1(), "%"G_GUINT64_FORMAT, &v)==1) {
        g_object_set(G_OBJECT(element), property_name, v, NULL);
      }
      break;
    }

    case G_TYPE_INT64 : {
      long long v;
      if(sscanf(qlb_number_value->text().latin1(), "%"G_GINT64_FORMAT, &v)==1) {
        g_object_set(G_OBJECT(element), property_name, v, NULL);
      }
      break;
    }

    case G_TYPE_FLOAT : {
      float v;
      if(sscanf(qlb_number_value->text().latin1(), "%g", &v)==1) {
        g_object_set(G_OBJECT(element), property_name, v, NULL);
      }
      break;
    }

    case G_TYPE_DOUBLE : {
      double v;
      if(sscanf(qlb_number_value->text().latin1(), "%lg", &v)==1) {
        g_object_set(G_OBJECT(element), property_name, v, NULL);
      }
      break;
    }
      
    default:
      if(G_IS_PARAM_SPEC_ENUM(param)) {
        QListBoxEnum *e=(QListBoxEnum*)qcmb_enum_value->listBox()->selectedItem();
        g_object_set(G_OBJECT(element), property_name, e->value, NULL);
      }
      break;
  }
  
  QDialog::accept();
}

void PropertyEditor::setNumberValue(const QString &value, const QString &hint, QValidator *validator) {
  qlb_number_value->setText(value);
  qlb_number_value->setValidator(validator);
  qlb_number_hint->setText(hint);
  ws_value->raiseWidget(2);
}
